@@ -33,7 +33,45 @@ $(function () {
       el.animate({
         opacity: 1
       });
+
+      // Copy to Clipboard buttons
+      // ZeroClipboard works by placing a transparent Flash
+      // embed over the element, which means that default hover
+      // and active states don't work, so we need to manually
+      // activate them. It uses its own .on() methods for events,
+      // so no clean jQuery syntax, unfortunately.
+      var copyBtn = new ZeroClipboard($('.copy-querystring'), { moviePath: '/static/flash/ZeroClipboard.swf' });
+
+      copyBtn.on('mousedown', function () {
+        $(this).addClass('active');
+      });
+      copyBtn.on('mouseup', function () {
+        $(this).removeClass('active');
+      });
+      copyBtn.on('mouseover', function () {
+        $(this).closest('tr').addClass('highlight');
+        $(this).tooltip('show');
+      });
+      copyBtn.on('mouseout', function () {
+        $(this).removeClass('active');
+        $(this).tooltip('hide');
+      });
+      copyBtn.on('complete', function (client, args) {
+        $('.copy-success').fadeIn().find('code').html(args.text);
+
+        // Take focus away from the flash doc
+        document.body.tabIndex = 0;
+        document.body.focus();
+      });
+
+      $('.copy-querystring').tooltip({ trigger: 'manual', placement: 'left' });
     });
+
+
+    // Focus the edit description textarea when opening the modal
+    $('#desc-modal').on('shown', function() {
+      $('#edit-description-textarea').focus();
+    })
   }
 
   // Draw charts on Dashboard page.
